/* ========================================
 *
 * 
 *
 * ========================================
*/

/* [] END OF FILE */



 
#include<device.h>
#include<herkulex.h>

void txPacket(uint8 packetSize, uint8* dat)
{
	int i;
	for(i=0;i<packetSize;i++)
    {
		UART_1_PutChar(*(dat+i));
   
       }

}
 
//------------------------------------------------------------------------------
void rxPacket(uint8 packetSize, uint8* dat)
{
 	int i;
	for(i=0;i<packetSize;i++)
		*(dat+i)=UART_1_GetChar();	
     
}
 
//------------------------------------------------------------------------------
void clear(uint8 id)
{
    uint8 txBuf[11];
    
    txBuf[0] = HEADER;              // Packet Header (0xFF)
    txBuf[1] = HEADER;              // Packet Header (0xFF)
    txBuf[2] = MIN_PACKET_SIZE + 4; // Packet Size
    txBuf[3] = id;                  // Servo ID
    txBuf[4] = CMD_RAM_WRITE;       // Command Ram Write (0x03)
    txBuf[5] = 0;                   // Checksum1
    txBuf[6] = 0;                   // Checksum2
    txBuf[7] = RAM_STATUS_ERROR;    // Address 48
    txBuf[8] = BYTE2;               // Length
    txBuf[9] = 0;                   // Clear RAM_STATUS_ERROR
    txBuf[10]= 0;                   // Clear RAM_STATUS_DETAIL
    
    // Checksum1 = (PacketSize ^ pID ^ CMD ^ dat[0] ^ dat[1] ^ ... ^ dat[n]) & 0xFE
    // Checksum2 = (~Checksum1)&0xFE
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]^txBuf[9]^txBuf[10]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(11, txBuf);
}
 
//------------------------------------------------------------------------------
void setTorque(uint8 id, uint8 cmdTorue)
{
    uint8 txBuf[10];
    
    txBuf[0] = HEADER;              // Packet Header (0xFF)
    txBuf[1] = HEADER;              // Packet Header (0xFF)
    txBuf[2] = MIN_PACKET_SIZE + 3; // Packet Size
    txBuf[3] = id;                  // Servo ID
    txBuf[4] = CMD_RAM_WRITE;       // Command Ram Write (0x03)
    txBuf[5] = 0;                   // Checksum1
    txBuf[6] = 0;                   // Checksum2
    txBuf[7] = RAM_TORQUE_CONTROL;  // Address 52
    txBuf[8] = BYTE1;               // Length
    txBuf[9] = cmdTorue;            // Torque ON
 
    // Checksum1 = (PacketSize ^ pID ^ CMD ^ dat[0] ^ dat[1] ^ ... ^ dat[n]) & 0xFE
    // Checksum2 = (~Checksum1)&0xFE
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]^txBuf[9]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(10, txBuf);
}
 
//------------------------------------------------------------------------------
void positionControl(uint8 id, uint16 position, uint8 playtime, uint8 setLED)
{
    uint8 txBuf[12];
    if (position > 1023) return;
    if (playtime > 255) return;
    
    
    
    txBuf[0]  = HEADER;                 // Packet Header (0xFF)
    txBuf[1]  = HEADER;                 // Packet Header (0xFF)
    txBuf[2]  = MIN_PACKET_SIZE + 5;    // Packet Size
    txBuf[3]  = MAX_PID;                // pID is total number of servos in the network (0 ~ 253)
    txBuf[4]  = CMD_S_JOG;              // Command S JOG (0x06)
    txBuf[5]  = 0;                      // Checksum1
    txBuf[6]  = 0;                      // Checksum2
    txBuf[7]  = playtime;               // Playtime
    txBuf[8]  = position & 0x00FF;      // Position (LSB, Least Significant Bit)
    txBuf[9]  =(position & 0xFF00) >> 8;// position (MSB, Most Significanct Bit)
    txBuf[10] = POS_MODE | setLED;      // Pos Mode and LED on/off
    txBuf[11] = id;                     // Servo ID
    
    // Checksum1 = (PacketSize ^ pID ^ CMD ^ dat[0] ^ dat[1] ^ ... ^ dat[n]) & 0xFE
    // Checksum2 = (~Checksum1)&0xFE
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]^txBuf[9]^txBuf[10]^txBuf[11]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(12, txBuf);
}
 
//------------------------------------------------------------------------------
void velocityControl(uint8 id, int16 speed, uint8 setLED)
{
    
    
    uint8 txBuf[12];
    
    txBuf[0]  = HEADER;                 // Packet Header (0xFF)
    txBuf[1]  = HEADER;                 // Packet Header (0xFF)
    txBuf[2]  = MIN_PACKET_SIZE + 5;    // Packet Size
    txBuf[3]  = MAX_PID;                // pID is total number of servos in the network (0 ~ 253)
    txBuf[4]  = CMD_S_JOG;              // Command S JOG (0x06)
    txBuf[5]  = 0;                      // Checksum1
    txBuf[6]  = 0;                      // Checksum2
    txBuf[7]  = 0;                      // Playtime, unmeaningful in turn mode
    txBuf[8]  = speed & 0x00FF;         // Speed (LSB, Least Significant Bit)
    txBuf[9]  =(speed & 0xFF00) >> 8;   // Speed (MSB, Most Significanct Bit)
    txBuf[10] = TURN_MODE | setLED;     // Turn Mode and LED on/off
    txBuf[11] = id;                     // Servo ID
    
    if (speed > 1023 || speed < -1023) return;
    
    // Checksum1 = (PacketSize ^ pID ^ CMD ^ dat[0] ^ dat[1] ^ ... ^ dat[n]) & 0xFE
    // Checksum2 = (~Checksum1)&0xFE
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]^txBuf[9]^txBuf[10]^txBuf[11]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(12, txBuf);
}
 
//------------------------------------------------------------------------------
int8 getStatus(uint8 id) 
{
    uint8 status;
    uint8 txBuf[7];
     uint8 rxBuf[9];
     uint8 chksum1 ; 
     uint8 chksum2;
    txBuf[0] = HEADER;                  // Packet Header (0xFF)
    txBuf[1] = HEADER;                  // Packet Header (0xFF)
    txBuf[2] = MIN_PACKET_SIZE;         // Packet Size
    txBuf[3] = id;                      // Servo ID
    txBuf[4] = CMD_STAT;                // Status Error, Status Detail request
 
    // Check Sum1 and Check Sum2
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(7, txBuf);
    
   CyDelay(10000);
   
    rxPacket(9, rxBuf);
 
    chksum1 = (rxBuf[2]^rxBuf[3]^rxBuf[4]^rxBuf[7]^rxBuf[8]) & 0xFE;

    // Checksum1
      
    if (chksum1 != rxBuf[5])
    {
        
        return -1;
    }
    
    // Checksum2
    chksum2 = (~rxBuf[5]&0xFE);
    if (chksum2 != rxBuf[6])
    {
       
        
        return -1;
    }
 
    status = rxBuf[7];  // Status Error
  //status = rxBuf[8];  // Status Detail
    
        
    
    return status;
}
 
//------------------------------------------------------------------------------
int getPos(uint8 id) 
{
    uint16 position = 0;
    uint8 chksum1;
    uint8 txBuf[9];
    uint8 rxBuf[13];
    uint8 chksum2;
    txBuf[0] = HEADER;                  // Packet Header (0xFF)
    txBuf[1] = HEADER;                  // Packet Header (0xFF)
    txBuf[2] = MIN_PACKET_SIZE + 2;     // Packet Size
    txBuf[3] = id;                      // Servo ID
    txBuf[4] = CMD_RAM_READ;            // Status Error, Status Detail request
    txBuf[5] = 0;                       // Checksum1
    txBuf[6] = 0;                       // Checksum2    
    txBuf[7] = RAM_CALIBRATED_POSITION; // Address 52
    txBuf[8] = BYTE2;                   // Address 52 and 53      
 
    // Check Sum1 and Check Sum2
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(9, txBuf);
    
    
    rxPacket(13, rxBuf);
 
    // Checksum1
     chksum1 = (rxBuf[2]^rxBuf[3]^rxBuf[4]^rxBuf[7]^rxBuf[8]^rxBuf[9]^rxBuf[10]^rxBuf[11]^rxBuf[12]) & 0xFE;    
    if (chksum1 != rxBuf[5])
    {
            
        
        return -1;
    }
    
    // Checksum2
     chksum2 = (~rxBuf[5]&0xFE);
    if (chksum2 != rxBuf[6])
    {
     
            
        
        return -1;
    }
 
    position = ((rxBuf[10]&0x03)<<8) | rxBuf[9];
    
    return position;
}


void movePos(uint8 id, uint16 pos, uint8 playtime, uint8 setMode, uint8 setLED)
{
    uint8 txBuf[12];
    if (pos > 1023) return;
    if (playtime > 255) return;
    
    
    
    txBuf[0]  = HEADER;                 // Packet Header (0xFF)
    txBuf[1]  = HEADER;                 // Packet Header (0xFF)
    txBuf[2]  = MIN_PACKET_SIZE + 5;    // Packet Size
    txBuf[3]  = MAX_PID;                // pID is total number of servos in the network (0 ~ 253)
    txBuf[4]  = CMD_S_JOG;              // Command S JOG (0x06)
    txBuf[5]  = 0;                      // Checksum1
    txBuf[6]  = 0;                      // Checksum2
    txBuf[7]  = playtime;               // Playtime
    txBuf[8]  = pos & 0x00FF;           // JOG(LSB)
    txBuf[9]  =(pos & 0xFF00) >> 8;     // JOG(MSB)
    txBuf[10] = setMode | setLED;       // Set mode and LED on/off
    txBuf[11] = id;                     // Servo ID
    
    // Checksum1 = (PacketSize ^ pID ^ CMD ^ Data[0] ^ Data[1] ^ ... ^ Data[n]) & 0xFE
    // Checksum2 = (~Checksum1)&0xFE
    txBuf[5] = (txBuf[2]^txBuf[3]^txBuf[4]^txBuf[7]^txBuf[8]^txBuf[9]^txBuf[10]^txBuf[11]) & 0xFE;
    txBuf[6] = (~txBuf[5])&0xFE;
 
    // send packet (mbed -> herkulex)
    txPacket(12, txBuf);
}
 




 
 
 
 

